require "qordoba_ruby_sdk/version"
require 'rest-client'
require 'json'

class QordobaRubySdk 
  
  # =========== Constant Vars ==============

  # API
  BASE = 'https://devapi.qordoba.com/v2'
  PING = 'ping'
  LANGUAGES = 'languages'
  COUNTRIES = 'countries'

  # ORGANIZATION
  ORG_TEAM = 'organizations/team'

  # PROJECTS
  PROJ_LIST = 'projects/list'
  PROJ_DETAIL = 'projects/detail'
  PROJ_STATUS = 'projects/status'
  PROJ_WORKFLOW = 'projects/workflow'
   
  # FILES
  FILE_LISTFILES = 'files/list'
  FILE_TYPES = 'files/types'
  FILE_UPLOAD = 'files/upload'
  FILE_EXPORT = 'files/export'
  FILE_UPDATE = 'files/update'
  FILE_SEGMENTS = 'files/segments/list'
  FILE_SEGMENT = 'files/segments/show'
  FILE_JSON = 'files/json'
  FILE_BULK1 = 'files/bulk1'
  FILE_BULK2 = 'files/bulk2'

  # TM
  TM_LIST = 'tms/list'
  TM_UPLOAD = 'tms/upload'
  TM_SHOW = 'tms/show'
  TM_SEGMENTS = 'tms/segments/list'
  TM_ADD_SEGMENT = 'tms/segments/add'
  TM_UPDATE_SEGMENT = 'tms/segments/update'
  TM_DELETE_SEGMENT = 'tms/segments/delete'

  # ERROR MESSAGES
  CONSUMER_KEY_ERROR = 'consumer key must be a 32 digit string'
  ORGANIZATION_ID_ERROR = 'organizationId must be a 4 digit number'
  PROJECT_ID_ERROR = 'projectId must be a 4 digit number'
  LANGUAGE_ID_ERROR = 'languageId must be an valid integer'
  FILE_IDS_ERROR = 'must be a valid fileId or array of valid fileIds'
  FILE_ID_ERROR = 'must be a valid six digit fileId'
  SEGMENT_ID_ERROR = 'must be a valid numberic segmentId'
  MILESTONE_ID_ERROR = 'must be a valid 4 digit numeric milestoneId'
  VERSION_TAG_ERROR = 'must pass a valid version tag string'
  PATH_TO_FILE_ERROR = 'must be a valid filepath'

  # =========== Initialize App ==============

  def initialize(consumerKey, organizationId, projectId)
    _validate({ 
      :consumerKey => consumerKey, 
      :organizationId => organizationId, 
      :projectId => projectId
    })

    @consumerKey = consumerKey
    @organizationId = organizationId
    @projectId = projectId

  end

  # ========= Validate User Inputs ============

  def _validate(hash)
    keys = hash.keys

    keys.each do |key|
      value = hash[key]
      if key == :consumerKey
        raise CONSUMER_KEY_ERROR unless value.kind_of?(String) && value.length == 32 
      elsif key == :organizationId
        raise ORGANIZATION_ID_ERROR unless value.is_a?(Integer) && value.to_s.length == 4
      elsif key == :projectId
        raise PROJECT_ID_ERROR unless value.is_a?(Integer) && value.to_s.length == 4
      elsif key == :languageId
        raise LANGUAGE_ID_ERROR unless value.is_a?(Integer) && value.to_s.length <= 3
      elsif key == :versionTag
        raise VERSION_TAG_ERROR unless value.is_a?(String)
      elsif key == :fileIds
        raise FILE_IDS_ERROR unless value.is_a?(Array) || value.is_a?(Integer)
      elsif key == :fileId
        raise FILE_ID_ERROR unless value.is_a?(Integer)
      elsif key == :segmentId
        raise SEGMENT_ID_ERROR unless value.is_a?(Integer)
      elsif key == :milestoneId
        raise MILESTONE_ID_ERROR unless value.is_a?(Integer) && value.to_s.length == 4
      elsif key == :pathToFile
        raise PATH_TO_FILE_ERROR unless File.exist?(value)
      end
    end
  end

  # ============= Parse JSON ================

  def _clean(response)
    if response.is_a?(String)
      JSON.parse(response)
    else
      response
    end
  end


  # =========== Getting Started ==============

  def ping
    _clean(RestClient.get "#{BASE}/#{PING}", 
      { :consumerKey => @consumerKey })
  end

  def languages
    _clean(RestClient.get "#{BASE}/#{LANGUAGES}", 
      { :consumerKey => @consumerKey })
  end

  def countries
    _clean(RestClient.get "#{BASE}/#{COUNTRIES}", 
      { :consumerKey => @consumerKey })
  end

  # ============= Organization ================

  def organization_team
    _clean(RestClient.get "#{BASE}/#{ORG_TEAM}", 
      { :consumerKey => @consumerKey,
        :organizationId => @organizationId })
  end

  # =============== Project ===================

  def project_list
    _clean(RestClient.get "#{BASE}/#{PROJ_LIST}", 
      { :consumerKey => @consumerKey,
        :organizationId => @organizationId })
  end

  def project_detail
    _clean(RestClient.get "#{BASE}/#{PROJ_DETAIL}", 
      { :consumerKey => @consumerKey,
        :projectId => @projectId })
  end

  def project_status
    _clean(RestClient.get "#{BASE}/#{PROJ_STATUS}", 
      { :consumerKey => @consumerKey,
        :projectId => @projectId })
  end

  def project_workflow
    _clean(RestClient.get "#{BASE}/#{PROJ_WORKFLOW}", 
      { :consumerKey => @consumerKey,
        :projectId => @projectId })
  end

  # ================ FILE ====================

  def file_list(languageId)
    _clean(RestClient.post "#{BASE}/#{FILE_LISTFILES}", {}.to_json,
      { :consumerKey => @consumerKey,
        :languageId => languageId,
        :projectId => @projectId,
        "content-type" => "application/json" })
  end

  def file_types
    _clean(RestClient.get "#{BASE}/#{FILE_TYPES}",
      { :consumerKey => @consumerKey,
        :projectId => @projectId })
  end

  def file_upload(pathToFile, versionTag)
    _clean(RestClient.post "#{BASE}/#{FILE_UPLOAD}",
      { :file_names => '[]', 
        :file => File.open(pathToFile), 
        :multipart => true
      }, 
      { :consumerKey => @consumerKey,
        :versionTag => versionTag,
        :projectId => @projectId,
        :organizationId => @organizationId,
        :params => { :type => "JSON" } })
  end

  def file_export(languageId, fileIds)
    if fileIds.is_a?(Array)
      fileIds = fileIds.join(',')
    end

    _clean(RestClient.post "#{BASE}/#{FILE_EXPORT}", {}.to_json,
      {
        :consumerKey => @consumerKey,
        :projectId => @projectId,
        :fileIds => fileIds,
        :targetLanguageIds => languageId,
        "content-type" => "application/json"
      })
  end

  def file_update(pathToFile, fileId)
    _clean(RestClient.post "#{BASE}/#{FILE_UPDATE}",
      { :file_names => '[]', 
        :file => File.open(pathToFile), 
        :multipart => true
      }, {
        :consumerKey => @consumerKey,
        :projectId => @projectId,
        :fileId => fileId,
        :params => { "type" => "JSON" }
      })
  end

  def file_segments(languageId, fileId)
    _clean(RestClient.get "#{BASE}/#{FILE_SEGMENTS}",
      { :consumerKey => @consumerKey,
        :projectId => @projectId,
        :languageId => languageId,
        :fileId => fileId })
  end

  def file_segment(languageId, fileId, segmentId)
    _clean(RestClient.get "#{BASE}/#{FILE_SEGMENT}",
      { :consumerKey => @consumerKey,
        :projectId => @projectId,
        :languageId => languageId,
        :fileId => fileId,
        :segmentId => segmentId })
  end

  def file_json(languageId, milestoneId, fileId)
    _clean(RestClient.get "#{BASE}/#{FILE_JSON}",
      { :consumerKey => @consumerKey,
        :projectId => @projectId,
        :languageId => languageId,
        :fileId => fileId,
        :milestoneId => milestoneId })
  end
end
